# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from unittest import TestCase
from restclients_core.exceptions import DataFailureException
from uw_space import Facilities
from uw_space.utils import fdao_space_override

data = {
    'code': 'MEB',
    'last_updated': '2022-09-22 19:41:34',
    'latitude': 47.6536929997,
    'longitude': -122.304747,
    'name': 'Mechanical Engineering Building',
    'number': '1347',
    'site': 'Seattle Main Campus',
    'type': 'Building'
    }


@fdao_space_override
class TestSpace(TestCase):
    def test_search_by_code(self):
        fac = Facilities().search_by_code("MEB")
        self.assertEqual(len(fac), 1)
        self.assertEqual(fac[0].json_data(), data)

        self.assertRaises(
            DataFailureException,
            Facilities().search_by_code, "None"
        )

    def test_search_by_number(self):
        fac = Facilities().search_by_number("1347")
        self.assertEqual(fac.json_data(), data)

        self.assertRaises(
            DataFailureException,
            Facilities().search_by_number, "0"
        )



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


# This is just a test runner for coverage
from commonconf.backends import use_configparser_backend
from os.path import abspath, dirname
import os


if __name__ == '__main__':
    path = abspath(os.path.join(dirname(__file__),
                                "..", "conf", "test.conf"))
    use_configparser_backend(path, 'SPACE')

    from nose2 import discover
    discover()



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


import json
import logging
from restclients_core.exceptions import DataFailureException
from uw_space.dao import SPACE_DAO
from uw_space.models import Facility

by_code_path = "/space/v2/facility.json?facility_code={}"
by_number_path = "/space/v2/facility/{}.json"
logger = logging.getLogger(__name__)


class Facilities(object):

    def __init__(self):
        self.dao = SPACE_DAO()
        self._read_headers = {
            'Accept': 'application/json',
            'Connection': 'keep-alive'}

    def search_by_code(self, facility_code):
        """
        facility_code: string
        """
        url = by_code_path.format(facility_code)
        response = self.dao.getURL(url, self._read_headers)
        logger.debug(
            {'url': url, 'status': response.status, 'data': response.data})
        if response.status != 200:
            raise DataFailureException(url, response.status, response.data)
        return self.__process_json(json.loads(response.data))

    def search_by_number(self, facility_number):
        """
        facility_number: string
        """
        url = by_number_path.format(facility_number)
        response = self.dao.getURL(url, self._read_headers)
        logger.debug(
            {'url': url, 'status': response.status, 'data': response.data})
        if response.status != 200:
            raise DataFailureException(url, response.status, response.data)
        return Facility.from_json(json.loads(response.data))

    def __process_json(self, json_data):
        objs = []
        facilitys = json_data.get("Facilitys")
        for facility in facilitys:
            fnumber = facility.get("FacilityNumber")
            if fnumber and len(fnumber):
                fac = self.search_by_number(fnumber)
                if fac:
                    objs.append(fac)
        return objs



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


import json
from restclients_core import models
from uw_space.utils import date_to_str, str_to_datetime


class Facility(models.Model):
    code = models.CharField(max_length=16)
    last_updated = models.DateTimeField()
    latitude = models.CharField(max_length=32)
    longitude = models.CharField(max_length=32)
    name = models.CharField(max_length=96)
    number = models.CharField(max_length=16)
    type = models.CharField(max_length=32)
    site = models.CharField(max_length=96)

    def __init__(self, *args, **kwargs):
        super(Facility, self).__init__(*args, **kwargs)

    @staticmethod
    def from_json(json_data):
        obj = Facility()
        obj.code = json_data.get("FacilityCode")
        obj.number = json_data.get("FacilityNumber")
        obj.last_updated = str_to_datetime(json_data.get("ModifiedDate"))
        obj.name = json_data.get("LongName")

        cpoint = json_data.get("CenterPoint")
        if cpoint:
            obj.latitude = cpoint.get("Latitude")
            obj.longitude = cpoint.get("Longitude")
        site_json = json_data.get("Site")

        if site_json:
            obj.site = site_json.get("Description")

        ftype = json_data.get("FacilityType")
        if ftype:
            obj.type = ftype.get("Description")
        return obj

    def json_data(self):
        return {
            "code": self.code,
            "last_updated": date_to_str(self.last_updated),
            "latitude": self.latitude,
            "longitude": self.longitude,
            "name": self.name,
            "number": self.number,
            "site": self.site,
            "type": self.type,
        }

    def __str__(self):
        return json.dumps(self.json_data(), default=str)



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from restclients_core.dao import DAO
from os.path import abspath, dirname
import os


class SPACE_DAO(DAO):
    def __init__(self):
        return super(SPACE_DAO, self).__init__()

    def service_name(self):
        return "space"

    def service_mock_paths(self):
        return [abspath(os.path.join(dirname(__file__), "resources"))]
